<?php

/**
 * Class WP_Query_BT_Location
 *
 * Extend \WP_Query to have custom data querying to get posts by lat/lng distance.
 */
class WP_Query_BT_Location extends \WP_Query {
	var $lat;
	var $lng;
	var $distance;
	//var $region;

	function __construct( $args = array() ) {
		foreach ( ['lat', 'lng', 'distance', /*'region'*/] as $arg ) {
			if ( ! empty( $args[$arg] ) ) {
				$this->{$arg} = $args[$arg];
			}
		}

		add_filter('posts_fields', array($this, 'posts_fields'));
		add_filter('posts_groupby', array($this, 'posts_groupby'));
		add_filter('posts_orderby', array($this, 'posts_orderby'));
		add_filter('posts_join_paged', array($this, 'posts_join_paged'));

		parent::query($args);

		remove_filter('posts_fields', array($this, 'posts_fields'));
		remove_filter('posts_groupby', array($this, 'posts_groupby'));
		remove_filter('posts_orderby', array($this, 'posts_orderby'));
		remove_filter('posts_join_paged', array($this, 'posts_join_paged'));
	}

	function posts_fields($fields) {
		global $wpdb;
		if ( $this->lat && $this->lng && $this->distance ) {
			// calculate distance
			$fields .= $wpdb->prepare( ", ((ACOS(SIN(%f * PI() / 180) * SIN(mt_lat.meta_value * PI() / 180) + COS(%f * PI() / 180) * COS(mt_lat.meta_value * PI() / 180) * COS((%f - mt_lng.meta_value) * PI() / 180)) * 180 / PI()) * 60 * 1.1515) AS distance", $this->lat, $this->lat, $this->lng );
			// include lat and lng for easy access
			$fields .= ', mt_lat.meta_value AS lat ';
			$fields .= ', mt_lng.meta_value AS lng ';
		}
		return $fields;
	}

	function posts_groupby($where) {
		global $wpdb;
		if ( $this->lat && $this->lng && $this->distance ) {
			/**
			 * NOTE: After removing the attributes/filters, we ended up with the sql being "GROUP BY  HAVING distance..."
			 * Not sure why, but adding the post ID column for it to group by makes it work.
			 */
			$where .= $wpdb->prepare( " wp_posts.ID HAVING distance < %d ", $this->distance );
		}
		return $where;
	}

	function posts_orderby($order) {
		//global $wpdb;
		$order = " distance ASC ";
		return $order;
	}

	function posts_join_paged($join) {
		//global $wpdb;
		if ( $this->lat && $this->lng && $this->distance ) {
			$join .= " INNER JOIN wp_postmeta AS mt_lat ON (wp_posts.ID = mt_lat.post_id AND mt_lat.meta_key = 'lat') ";
			$join .= " INNER JOIN wp_postmeta AS mt_lng ON (wp_posts.ID = mt_lng.post_id AND mt_lng.meta_key = 'lng') ";
		}
		/* if ( $this->region && is_numeric( $this->region ) ) {
			$join .= $wpdb->prepare( " INNER JOIN wp_postmeta AS mt_region ON (wp_posts.ID = mt_region.post_id AND mt_region.meta_key = 'region' AND mt_region.meta_value = %d) ", $this->region );
		} */
		return $join;
	}
}
